#include <Wire.h>
#include <LiquidCrystal_I2C.h>

// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // setting LCD I2C address

//set pin number
const int buzzerPin = 1;
const int ledPin = 13;
const int buttonPin = 2;

//set variables for record results
int expired_time = 1000;//set expired time for player with no reaction
long randomTimer; //set LED and buzzer random turn on time variable holder
int time1, time2, finalTime; //variable for player 1, player 2 reaction time comparison
unsigned long startFirst, endFirst; //variable for tracking player's reaction time
unsigned long  startSecond, endSecond; //variable for tracking player's reaction time                            
String winner="";   //variable for store winner

void setup() {
  
  lcd.begin(16, 2);
  pinMode(buzzerPin, OUTPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(buttonPin, INPUT);  
  randomSeed(analogRead(0));  
  
  for(int i = 0; i < 3; i++) {   // flshing backlight 3 times
    lcd.backlight();
    delay(250);
    lcd.noBacklight();
    delay(250);
  }
  lcd.backlight();

  lcd.clear();    // clear LCD dislay screen
  lcd.setCursor(0,0);
  lcd.print("  Welcome to  ");  //print welcome message
  lcd.setCursor(0,1);
  lcd.print(" Reaction Game ");
  delay(4000);
}

void loop() {

  lcd.clear();    
  lcd.setCursor(0,0);
  lcd.print("Press Button");    // give instruction to user
  lcd.setCursor(0,1);
  lcd.print("on beep");
  delay(4000);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Best of 2");
  lcd.setCursor(0,1);
  lcd.print("tries wins");
  delay(4000);

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Get Ready!!");   //ask 1st Player to get ready
  lcd.setCursor(0,1);
  lcd.print("First Player");
  delay(3000);
  lcd.clear();
    
  randomTimer = random(100, 6000);  //set random time range
  delay(randomTimer);         
  digitalWrite(ledPin, HIGH);       //turn on LED
  digitalWrite(buzzerPin, HIGH);    //turn on buzzer
  tone(buzzerPin, 1568);
  
  startFirst = millis();      //set initial time counter for tracking reaction time
  while(millis() - startFirst < expired_time && digitalRead(buttonPin)!=HIGH){
    delay(1); //every millisecond we check the button pressed or not
  }

  endFirst = millis() - startFirst;   //get the reaction time 
                                      // by subtracting initial time by current time 
  digitalWrite(ledPin, LOW);
  noTone(buzzerPin);  
  time1 = endFirst;   //set the player 1 reaction variable as recorded reaction time 

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("First Reaction:");
  lcd.setCursor(0,1);
  lcd.print(time1);
  lcd.print(" ms");
  delay(3000);
  lcd.clear();

//2nd Player
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Get Ready!!");
  lcd.setCursor(0,1);
  lcd.print("Second Player");
  delay(3000);
  lcd.clear();
    
  randomTimer = random(100, 6000);
  delay(randomTimer);
  digitalWrite(ledPin, HIGH);
  tone(buzzerPin, 1568);
  
  startSecond = millis();
  while(millis() - startSecond < expired_time && digitalRead(buttonPin)!=HIGH){
    delay(1); //every millisecond we check the button pressed or not
  }

  endSecond = millis() - startSecond;
  digitalWrite(ledPin, LOW);
  noTone(buzzerPin);
  time2 = endSecond;

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Second Reaction:");
  lcd.setCursor(0,1);
  lcd.print(time2);
  lcd.print(" ms");
  delay(3000);
  lcd.clear();

  if(time1 > time2){        //Compare time
    finalTime = time2;
    winner = "Player 2";
  }else{
    finalTime = time1;  
    winner = "Player 1";    
  }

  lcd.setCursor(0,0);     //Display Winner
  lcd.print("Winner:");
  lcd.print(winner);  
  lcd.setCursor(0,1);
  lcd.print(finalTime);
  lcd.print(" ms");
  delay(3000); 

  lcd.clear();            //Restart message
  lcd.setCursor(0,0);
  lcd.print("Well done!");
  lcd.setCursor(0,1);
  lcd.print("Let's Play Again");  
  delay(3000);
  
}
